@@ -1,7 +1,7 @@
 /*
- *  This file is part of nzbget
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2007-2015 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,29 +14,25 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- *
- * $Revision$
- * $Date$
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
 #ifndef NZBSCRIPT_H
 #define NZBSCRIPT_H
 
-#include "Script.h"
+#include "ScriptController.h"
 #include "DownloadInfo.h"
-#include "ScriptConfig.h"
+#include "Extension.h"
+#include "ExtensionManager.h"
 
-class NZBScriptController : public ScriptController
+class NzbScriptController : public ScriptController
 {
 protected:
-	void				PrepareEnvParameters(NZBParameterList* pParameters, const char* szStripPrefix);
-	void				PrepareEnvScript(NZBParameterList* pParameters, const char* szScriptName);
-	void				ExecuteScriptList(const char* szScriptList);
-	virtual void		ExecuteScript(ScriptConfig::Script* pScript) = 0;
+	void PrepareEnvParameters(NzbParameterList* parameters, const char* stripPrefix);
+	void PrepareEnvScript(NzbParameterList* parameters, const char* scriptName);
+	void ExecuteScriptList(const char* scriptList);
+	virtual void ExecuteScript(std::shared_ptr<const Extension::Script> script) = 0;
 };
 
 #endif
