@@ -2,6 +2,7 @@
  *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2013-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2025 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +22,8 @@
 #ifndef FEEDFILE_H
 #define FEEDFILE_H
 
-#include "NString.h"
+#include <string>
+#include <string_view>
 #include "FeedInfo.h"
 
 class FeedFile
@@ -35,23 +37,30 @@ public:
 
 private:
 	std::unique_ptr<FeedItemList> m_feedItems;
-	CString m_fileName;
-	CString m_infoName;
+	std::string m_fileName;
+	std::string m_infoName;
 
 	void ParseSubject(FeedItemInfo& feedItemInfo);
 
 	FeedItemInfo* m_feedItemInfo;
-	StringBuilder m_tagContent;
+	std::string m_tagContent;
 	bool m_ignoreNextError;
 
 	static void SAX_StartElement(FeedFile* file, const char *name, const char **atts);
 	static void SAX_EndElement(FeedFile* file, const char *name);
-	static void SAX_characters(FeedFile* file, const char *  xmlstr, int len);
-	static void* SAX_getEntity(FeedFile* file, const char *  name);
+	static void SAX_textHandler(FeedFile* file, const char *  xmlstr, int len);
+	static xmlEntityPtr SAX_getEntity(FeedFile* file, const xmlChar*  name);
 	static void SAX_error(FeedFile* file, const char *msg, ...);
 	void Parse_StartElement(const char *name, const char **atts);
 	void Parse_EndElement(const char *name);
-	void Parse_Content(const char *buf, int len);
+	void Parse_Content(std::string content);
+
+	/**
+	 * Extracts the file size in bytes from a description string.
+	 * Some RSS feeds embed the file size directly within the description text.
+	 * Example description string: "Total Size : 299.6 MB | MultiUp"
+	 */
+	int64 ExtractSizeFromDescription(std::string_view description);
 	void ResetTagContent();
 };
 
