@@ -1,5 +1,5 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
  *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
@@ -34,6 +34,10 @@ NntpConnection::NntpConnection(NewsServer* newsServer) :
 	SetCipher(newsServer->GetCipher());
 	SetIPVersion(newsServer->GetIpVersion() == 4 ? Connection::ipV4 :
 		newsServer->GetIpVersion() == 6 ? Connection::ipV6 : Connection::ipAuto);
+	
+#ifndef DISABLE_TLS
+	SetCertVerifLevel(newsServer->GetCertVerificationLevel());
+#endif
 }
 
 const char* NntpConnection::Request(const char* req)
@@ -227,10 +231,7 @@ bool NntpConnection::Disconnect()
 {
 	if (m_status == csConnected)
 	{
-		if (!m_broken)
-		{
-			Request("quit\r\n");
-		}
+		Request("quit\r\n");
 		m_activeGroup = nullptr;
 	}
 	return Connection::Disconnect();
