@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2007-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -33,7 +34,7 @@ bool ParParser::FindMainPars(const char* path, ParFileList* fileList)
 	while (const char* filename = dir.Next())
 	{
 		int baseLen = 0;
-		if (ParseParFilename(filename, &baseLen, nullptr))
+		if (ParseParFilename(filename, true, &baseLen, nullptr))
 		{
 			if (!fileList)
 			{
@@ -42,9 +43,9 @@ bool ParParser::FindMainPars(const char* path, ParFileList* fileList)
 
 			// check if the base file already added to list
 			bool exists = false;
-			for (CString& filename2 : fileList)
+			for (std::string& filename2 : *fileList)
 			{
-				exists = SameParCollection(filename, filename2);
+				exists = SameParCollection(filename, filename2.c_str(), true);
 				if (exists)
 				{
 					break;
@@ -59,16 +60,16 @@ bool ParParser::FindMainPars(const char* path, ParFileList* fileList)
 	return fileList && !fileList->empty();
 }
 
-bool ParParser::SameParCollection(const char* filename1, const char* filename2)
+bool ParParser::SameParCollection(const char* filename1, const char* filename2, bool confirmedFilenames)
 {
 	int baseLen1 = 0, baseLen2 = 0;
-	return ParseParFilename(filename1, &baseLen1, nullptr) &&
-		ParseParFilename(filename2, &baseLen2, nullptr) &&
+	return ParseParFilename(filename1, confirmedFilenames, &baseLen1, nullptr) &&
+		ParseParFilename(filename2, confirmedFilenames, &baseLen2, nullptr) &&
 		baseLen1 == baseLen2 &&
 		!strncasecmp(filename1, filename2, baseLen1);
 }
 
-bool ParParser::ParseParFilename(const char* parFilename, int* baseNameLen, int* blocks)
+bool ParParser::ParseParFilename(const char* parFilename, bool confirmedFilename, int* baseNameLen, int* blocks)
 {
 	BString<1024> filename = parFilename;
 	for (char* p = filename; *p; p++) *p = tolower(*p); // convert string to lowercase
@@ -79,10 +80,13 @@ bool ParParser::ParseParFilename(const char* parFilename, int* baseNameLen, int*
 		return false;
 	}
 
-	// find last occurence of ".par2" and trim filename after it
-	char* end = filename;
-	while (char* p = strstr(end, ".par2")) end = p + 5;
-	*end = '\0';
+	if (!confirmedFilename)
+	{
+		// find last occurence of ".par2" and trim filename after it
+		char* end = filename;
+		while (char* p = strstr(end, ".par2")) end = p + 5;
+		*end = '\0';
+	}
 
 	len = strlen(filename);
 	if (len < 6)
