@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2024 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,13 +15,14 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 
 #include "nzbget.h"
 #include "PrePostProcessor.h"
 #include "Options.h"
+#include "WorkState.h"
 #include "Log.h"
 #include "HistoryCoordinator.h"
 #include "DupeCoordinator.h"
@@ -35,6 +37,7 @@
 #include "QueueScript.h"
 #include "ParParser.h"
 #include "DirectUnpack.h"
+#include "PostUnpackRenamer.h"
 
 PrePostProcessor::PrePostProcessor()
 {
@@ -49,23 +52,35 @@ void PrePostProcessor::Run()
 
 	while (!DownloadQueue::IsLoaded())
 	{
-		usleep(20 * 1000);
+		Util::Sleep(20);
 	}
 
-	if (g_Options->GetServerMode() && g_Options->GetSaveQueue() && g_Options->GetReloadQueue())
+	if (g_Options->GetServerMode())
 	{
 		SanitisePostQueue();
 	}
 
 	while (!IsStopped())
 	{
-		if (!g_Options->GetTempPausePostprocess() && m_queuedJobs)
+		if (g_WorkState->GetTempPausePostprocess())
+		{
+			// Postprocess is paused: just wait and loop
+			Util::Sleep(200);
+			continue;
+		}
+
+		if (m_queuedJobs)
 		{
 			// check post-queue every 200 msec
 			CheckPostQueue();
+			Util::Sleep(200);
+		}
+		else
+		{
+			// Wait until we get the stop signal or more jobs in the queue
+			Guard guard(m_waitMutex);
+			m_waitCond.Wait(m_waitMutex, [&]{ return m_queuedJobs || IsStopped(); });
 		}
-
-		usleep(200 * 1000);
 	}
 
 	WaitJobs();
@@ -89,7 +104,7 @@ void PrePostProcessor::WaitJobs()
 			}
 		}
 		CheckPostQueue();
-		usleep(200 * 1000);
+		Util::Sleep(200);
 	}
 
 	// kill remaining post-processing jobs; not safe but we can't wait any longer
@@ -124,7 +139,7 @@ void PrePostProcessor::WaitJobs()
 				break;
 			}
 		}
-		usleep(200 * 1000);
+		Util::Sleep(200);
 	}
 
 	// disconnect remaining direct unpack jobs
@@ -142,23 +157,30 @@ void PrePostProcessor::WaitJobs()
 void PrePostProcessor::Stop()
 {
 	Thread::Stop();
-	GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
 
-	for (NzbInfo* postJob : m_activeJobs)
 	{
-		if (postJob->GetPostInfo() && postJob->GetPostInfo()->GetPostThread())
+		GuardedDownloadQueue downloadQueue = DownloadQueue::Guard();
+
+		for (NzbInfo* postJob : m_activeJobs)
 		{
-			postJob->GetPostInfo()->GetPostThread()->Stop();
+			if (postJob->GetPostInfo() && postJob->GetPostInfo()->GetPostThread())
+			{
+				postJob->GetPostInfo()->GetPostThread()->Stop();
+			}
+		}
+
+		for (NzbInfo* nzbInfo : downloadQueue->GetQueue())
+		{
+			if (nzbInfo->GetUnpackThread())
+			{
+				((DirectUnpack*)nzbInfo->GetUnpackThread())->Stop(downloadQueue, nzbInfo);
+			}
 		}
 	}
 
-	for (NzbInfo* nzbInfo : downloadQueue->GetQueue())
-	{
-		if (nzbInfo->GetUnpackThread())
-		{
-			((DirectUnpack*)nzbInfo->GetUnpackThread())->Stop(downloadQueue, nzbInfo);
-		}
-	}
+	// Resume Run() to exit it
+	Guard guard(m_waitMutex);
+	m_waitCond.NotifyAll();
 }
 
 /**
@@ -197,11 +219,13 @@ void PrePostProcessor::DownloadQueueUpdate(void* aspect)
 	}
 
 	DownloadQueue::Aspect* queueAspect = (DownloadQueue::Aspect*)aspect;
-	if (queueAspect->action == DownloadQueue::eaNzbFound)
+	if (queueAspect->action == DownloadQueue::eaNzbFound ||
+		queueAspect->action == DownloadQueue::eaUrlFound)
 	{
 		NzbFound(queueAspect->downloadQueue, queueAspect->nzbInfo);
 	}
-	else if (queueAspect->action == DownloadQueue::eaNzbAdded)
+	else if (queueAspect->action == DownloadQueue::eaNzbAdded ||
+		queueAspect->action == DownloadQueue::eaUrlAdded)
 	{
 		NzbAdded(queueAspect->downloadQueue, queueAspect->nzbInfo);
 	}
@@ -220,6 +244,14 @@ void PrePostProcessor::DownloadQueueUpdate(void* aspect)
 			"Collection %s deleted from queue", queueAspect->nzbInfo->GetName());
 		NzbDeleted(queueAspect->downloadQueue, queueAspect->nzbInfo);
 	}
+	else if (queueAspect->action == DownloadQueue::eaUrlDeleted)
+	{
+		NzbDeleted(queueAspect->downloadQueue, queueAspect->nzbInfo);
+	}
+	else if (queueAspect->action == DownloadQueue::eaUrlFailed)
+	{
+		NzbCompleted(queueAspect->downloadQueue, queueAspect->nzbInfo, true);
+	}
 	else if ((queueAspect->action == DownloadQueue::eaFileCompleted ||
 		queueAspect->action == DownloadQueue::eaFileDeleted))
 	{
@@ -303,18 +335,30 @@ void PrePostProcessor::NzbAdded(DownloadQueue* downloadQueue, NzbInfo* nzbInfo)
 
 void PrePostProcessor::NzbDownloaded(DownloadQueue* downloadQueue, NzbInfo* nzbInfo)
 {
+	if (!nzbInfo)
+	{
+		return;
+	}
+
+	if (nzbInfo->GetSkipScriptProcessing() && nzbInfo->GetSkipDiskWrite())
+	{
+		NzbCompleted(downloadQueue, nzbInfo, true);
+		nzbInfo->SetCleanupDisk(true);
+		DeleteCleanup(nzbInfo);
+		return;
+	}
+
 	if (nzbInfo->GetDeleteStatus() == NzbInfo::dsHealth ||
 		nzbInfo->GetDeleteStatus() == NzbInfo::dsBad)
 	{
 		g_QueueScriptCoordinator->EnqueueScript(nzbInfo, QueueScriptCoordinator::qeNzbDeleted);
 	}
 
-	if (!nzbInfo->GetPostInfo() && g_Options->GetDecode())
+	if (!nzbInfo->GetPostInfo() && !g_Options->GetRawArticle() && !g_Options->GetSkipWrite() && !nzbInfo->GetSkipDiskWrite())
 	{
 		nzbInfo->PrintMessage(Message::mkInfo, "Queueing %s for post-processing", nzbInfo->GetName());
 
 		nzbInfo->EnterPostProcess();
-		m_queuedJobs++;
 
 		if (nzbInfo->GetParStatus() == NzbInfo::psNone &&
 			g_Options->GetParCheck() != Options::pcAlways &&
@@ -330,7 +374,13 @@ void PrePostProcessor::NzbDownloaded(DownloadQueue* downloadQueue, NzbInfo* nzbI
 			((DirectUnpack*)nzbInfo->GetUnpackThread())->NzbDownloaded(downloadQueue, nzbInfo);
 		}
 
-		downloadQueue->Save();
+		nzbInfo->SetChanged(true);
+		downloadQueue->SaveChanged();
+
+		// We have more jobs in the queue, notify Run()
+		Guard guard(m_waitMutex);
+		m_queuedJobs++;
+		m_waitCond.NotifyAll();
 	}
 	else
 	{
@@ -371,6 +421,7 @@ void PrePostProcessor::NzbDeleted(DownloadQueue* downloadQueue, NzbInfo* nzbInfo
 
 void PrePostProcessor::NzbCompleted(DownloadQueue* downloadQueue, NzbInfo* nzbInfo, bool saveQueue)
 {
+	bool downloadDupe = nzbInfo->GetDupeHint() == NzbInfo::dhRedownloadAuto;
 	bool addToHistory = g_Options->GetKeepHistory() > 0 && !nzbInfo->GetAvoidHistory();
 	if (addToHistory)
 	{
@@ -384,7 +435,8 @@ void PrePostProcessor::NzbCompleted(DownloadQueue* downloadQueue, NzbInfo* nzbIn
 		(nzbInfo->GetDeleteStatus() == NzbInfo::dsNone ||
 		 nzbInfo->GetDeleteStatus() == NzbInfo::dsHealth ||
 		 nzbInfo->GetDeleteStatus() == NzbInfo::dsBad ||
-		 nzbInfo->GetDeleteStatus() == NzbInfo::dsScan))
+		 nzbInfo->GetDeleteStatus() == NzbInfo::dsScan ||
+		 (nzbInfo->GetDeleteStatus() == NzbInfo::dsCopy && downloadDupe)))
 	{
 		g_DupeCoordinator->NzbCompleted(downloadQueue, nzbInfo);
 		needSave = true;
@@ -415,6 +467,18 @@ void PrePostProcessor::DeleteCleanup(NzbInfo* nzbInfo)
 	if (nzbInfo->GetCleanupDisk() ||
 		nzbInfo->GetDeleteStatus() == NzbInfo::dsDupe)
 	{
+		if (nzbInfo->GetSkipDiskWrite())
+		{
+			CString errmsg;
+			detail("Deleting dir %s", nzbInfo->GetDestDir());
+			if (!FileSystem::DeleteDirectoryWithContent(nzbInfo->GetDestDir(), errmsg))
+			{
+				error("Could not delete directory %s: %s", nzbInfo->GetDestDir(), *errmsg);
+			}
+
+			return;
+		}
+
 		// download was cancelled, deleting already downloaded files from disk
 		for (CompletedFile& completedFile: nzbInfo->GetCompletedFiles())
 		{
@@ -426,12 +490,12 @@ void PrePostProcessor::DeleteCleanup(NzbInfo* nzbInfo)
 			}
 		}
 
-		// delete .out.tmp-files and _brokenlog.txt
+		// delete .out.tmp-files
 		DirBrowser dir(nzbInfo->GetDestDir());
 		while (const char* filename = dir.Next())
 		{
 			int len = strlen(filename);
-			if ((len > 8 && !strcmp(filename + len - 8, ".out.tmp")) || !strcmp(filename, "_brokenlog.txt"))
+			if (len > 8 && !strcmp(filename + len - 8, ".out.tmp"))
 			{
 				BString<1024> fullFilename("%s%c%s", nzbInfo->GetDestDir(), PATH_SEPARATOR, filename);
 				detail("Deleting file %s", filename);
@@ -574,7 +638,7 @@ NzbInfo* PrePostProcessor::PickNextJob(DownloadQueue* downloadQueue, bool allowP
 			!g_QueueScriptCoordinator->HasJob(nzbInfo1->GetId(), nullptr) &&
 			nzbInfo1->GetDirectUnpackStatus() != NzbInfo::nsRunning &&
 			(!nzbInfo || nzbInfo1->GetPriority() > nzbInfo->GetPriority()) &&
-			(!g_Options->GetPausePostProcess() || nzbInfo1->GetForcePriority()) &&
+			(!g_WorkState->GetPausePostProcess() || nzbInfo1->GetForcePriority()) &&
 			(allowPar || !nzbInfo1->GetPostInfo()->GetNeedParCheck()) &&
 			(std::find(m_activeJobs.begin(), m_activeJobs.end(), nzbInfo1) == m_activeJobs.end()) &&
 			nzbInfo1->IsDownloadCompleted(true))
@@ -608,7 +672,7 @@ void PrePostProcessor::CheckPostQueue()
 
 		PostInfo* postInfo = postJob->GetPostInfo();
 		if (postInfo->GetStage() == PostInfo::ptQueued &&
-			(!g_Options->GetPausePostProcess() || postInfo->GetNzbInfo()->GetForcePriority()))
+			(!g_WorkState->GetPausePostProcess() || postInfo->GetNzbInfo()->GetForcePriority()))
 		{
 			StartJob(downloadQueue, postInfo, allowPar);
 			CheckRequestPar(downloadQueue);
@@ -628,6 +692,8 @@ void PrePostProcessor::CheckPostQueue()
 
 void PrePostProcessor::StartJob(DownloadQueue* downloadQueue, PostInfo* postInfo, bool allowPar)
 {
+	NzbInfo* nzbInfo = postInfo->GetNzbInfo();
+
 	if (!postInfo->GetStartTime())
 	{
 		postInfo->SetStartTime(Util::CurrentTime());
@@ -637,8 +703,8 @@ void PrePostProcessor::StartJob(DownloadQueue* downloadQueue, PostInfo* postInfo
 	postInfo->SetFileProgress(0);
 	postInfo->SetProgressLabel("");
 
-	if (postInfo->GetNzbInfo()->GetParRenameStatus() == NzbInfo::rsNone &&
-		postInfo->GetNzbInfo()->GetDeleteStatus() == NzbInfo::dsNone &&
+	if (nzbInfo->GetParRenameStatus() == NzbInfo::rsNone &&
+		nzbInfo->GetDeleteStatus() == NzbInfo::dsNone &&
 		g_Options->GetParRename())
 	{
 		EnterStage(downloadQueue, postInfo, PostInfo::ptParRenaming);
@@ -647,10 +713,10 @@ void PrePostProcessor::StartJob(DownloadQueue* downloadQueue, PostInfo* postInfo
 	}
 
 #ifndef DISABLE_PARCHECK
-	if (postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psNone &&
-		postInfo->GetNzbInfo()->GetDeleteStatus() == NzbInfo::dsNone)
+	if (nzbInfo->GetParStatus() == NzbInfo::psNone &&
+		nzbInfo->GetDeleteStatus() == NzbInfo::dsNone)
 	{
-		if (ParParser::FindMainPars(postInfo->GetNzbInfo()->GetDestDir(), nullptr))
+		if (ParParser::FindMainPars(nzbInfo->GetDestDir(), nullptr))
 		{
 			if (!allowPar)
 			{
@@ -664,54 +730,54 @@ void PrePostProcessor::StartJob(DownloadQueue* downloadQueue, PostInfo* postInfo
 		}
 		else
 		{
-			postInfo->GetNzbInfo()->PrintMessage(Message::mkInfo,
-				"Nothing to par-check for %s", postInfo->GetNzbInfo()->GetName());
-			postInfo->GetNzbInfo()->SetParStatus(NzbInfo::psSkipped);
+			nzbInfo->PrintMessage(Message::mkInfo,
+				"Nothing to par-check for %s", nzbInfo->GetName());
+			nzbInfo->SetParStatus(NzbInfo::psSkipped);
 		}
 		return;
 	}
 	
-	if (postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psSkipped &&
+	if (nzbInfo->GetParStatus() == NzbInfo::psSkipped &&
 		((g_Options->GetParScan() != Options::psDupe &&
-		  postInfo->GetNzbInfo()->CalcHealth() < postInfo->GetNzbInfo()->CalcCriticalHealth(false) &&
-		  postInfo->GetNzbInfo()->CalcCriticalHealth(false) < 1000) ||
-		  postInfo->GetNzbInfo()->CalcHealth() == 0) &&
-		ParParser::FindMainPars(postInfo->GetNzbInfo()->GetDestDir(), nullptr))
+		  nzbInfo->CalcHealth() < nzbInfo->CalcCriticalHealth(false) &&
+		  nzbInfo->CalcCriticalHealth(false) < 1000) ||
+		  nzbInfo->CalcHealth() == 0) &&
+		ParParser::FindMainPars(nzbInfo->GetDestDir(), nullptr))
 	{
-		if (postInfo->GetNzbInfo()->CalcHealth() == 0)
+		if (nzbInfo->CalcHealth() == 0)
 		{
-			postInfo->GetNzbInfo()->PrintMessage(Message::mkWarning,
-				"Skipping par-check for %s due to health 0%%", postInfo->GetNzbInfo()->GetName());
+			nzbInfo->PrintMessage(Message::mkWarning,
+				"Skipping par-check for %s due to health 0%%", nzbInfo->GetName());
 		}
 		else
 		{
-			postInfo->GetNzbInfo()->PrintMessage(Message::mkWarning,
+			nzbInfo->PrintMessage(Message::mkWarning,
 				"Skipping par-check for %s due to health %.1f%% below critical %.1f%%",
-				postInfo->GetNzbInfo()->GetName(),
-				postInfo->GetNzbInfo()->CalcHealth() / 10.0, postInfo->GetNzbInfo()->CalcCriticalHealth(false) / 10.0);
+				nzbInfo->GetName(),
+				nzbInfo->CalcHealth() / 10.0, nzbInfo->CalcCriticalHealth(false) / 10.0);
 		}
-		postInfo->GetNzbInfo()->SetParStatus(NzbInfo::psFailure);
+		nzbInfo->SetParStatus(NzbInfo::psFailure);
 		return;
 	}
 	
-	if (postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psSkipped &&
-		postInfo->GetNzbInfo()->GetFailedSize() - postInfo->GetNzbInfo()->GetParFailedSize() > 0 &&
-		ParParser::FindMainPars(postInfo->GetNzbInfo()->GetDestDir(), nullptr))
+	if (nzbInfo->GetParStatus() == NzbInfo::psSkipped &&
+		nzbInfo->GetFailedSize() - nzbInfo->GetParFailedSize() > 0 &&
+		ParParser::FindMainPars(nzbInfo->GetDestDir(), nullptr))
 	{
-		postInfo->GetNzbInfo()->PrintMessage(Message::mkInfo,
+		nzbInfo->PrintMessage(Message::mkInfo,
 			"Collection %s with health %.1f%% needs par-check",
-			postInfo->GetNzbInfo()->GetName(), postInfo->GetNzbInfo()->CalcHealth() / 10.0);
+			nzbInfo->GetName(), nzbInfo->CalcHealth() / 10.0);
 		postInfo->SetRequestParCheck(true);
 		return;
 	}
 #endif
 
-	NzbParameter* unpackParameter = postInfo->GetNzbInfo()->GetParameters()->Find("*Unpack:", false);
+	NzbParameter* unpackParameter = nzbInfo->GetParameters()->Find("*Unpack:");
 	bool wantUnpack = !(unpackParameter && !strcasecmp(unpackParameter->GetValue(), "no"));
-	bool unpack = wantUnpack && postInfo->GetNzbInfo()->GetUnpackStatus() == NzbInfo::usNone &&
-		postInfo->GetNzbInfo()->GetDeleteStatus() == NzbInfo::dsNone;
+	bool unpack = wantUnpack && nzbInfo->GetUnpackStatus() == NzbInfo::usNone &&
+		nzbInfo->GetDeleteStatus() == NzbInfo::dsNone;
 
-	if (postInfo->GetNzbInfo()->GetRarRenameStatus() == NzbInfo::rsNone &&
+	if (nzbInfo->GetRarRenameStatus() == NzbInfo::rsNone &&
 		unpack && g_Options->GetRarRename())
 	{
 		EnterStage(downloadQueue, postInfo, PostInfo::ptRarRenaming);
@@ -719,46 +785,77 @@ void PrePostProcessor::StartJob(DownloadQueue* downloadQueue, PostInfo* postInfo
 		return;
 	}
 
-	bool parFailed = postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psFailure ||
-		postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psRepairPossible ||
-		postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psManual;
+#ifndef DISABLE_PARCHECK
+	if (nzbInfo->GetParStatus() == NzbInfo::psSkipped &&
+		nzbInfo->GetDeleteStatus() == NzbInfo::dsNone &&
+		g_Options->GetParCheck() == Options::pcAuto &&
+		!UnpackController::HasCompletedArchiveFiles(nzbInfo) &&
+		ParParser::FindMainPars(nzbInfo->GetDestDir(), nullptr))
+	{
+		nzbInfo->PrintMessage(Message::mkInfo,
+			"Requesting par-check for collection %s without archive files",
+			nzbInfo->GetName());
+		postInfo->SetRequestParCheck(true);
+		return;
+	}
+#endif
+
+	bool parFailed = nzbInfo->GetParStatus() == NzbInfo::psFailure ||
+		nzbInfo->GetParStatus() == NzbInfo::psRepairPossible ||
+		nzbInfo->GetParStatus() == NzbInfo::psManual;
 
 	bool cleanup = !unpack && wantUnpack &&
-		postInfo->GetNzbInfo()->GetCleanupStatus() == NzbInfo::csNone &&
+		nzbInfo->GetCleanupStatus() == NzbInfo::csNone &&
 		!Util::EmptyStr(g_Options->GetExtCleanupDisk()) &&
-		((postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psSuccess &&
-		  postInfo->GetNzbInfo()->GetUnpackStatus() != NzbInfo::usFailure &&
-		  postInfo->GetNzbInfo()->GetUnpackStatus() != NzbInfo::usSpace &&
-		  postInfo->GetNzbInfo()->GetUnpackStatus() != NzbInfo::usPassword) ||
-		 (postInfo->GetNzbInfo()->GetUnpackStatus() == NzbInfo::usSuccess &&
-		  postInfo->GetNzbInfo()->GetParStatus() != NzbInfo::psFailure) ||
-		 ((postInfo->GetNzbInfo()->GetUnpackStatus() == NzbInfo::usNone ||
-		   postInfo->GetNzbInfo()->GetUnpackStatus() == NzbInfo::usSkipped) &&
-		  (postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psNone ||
-		   postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psSkipped) &&
-		  postInfo->GetNzbInfo()->CalcHealth() == 1000));
+		((nzbInfo->GetParStatus() == NzbInfo::psSuccess &&
+		  nzbInfo->GetUnpackStatus() != NzbInfo::usFailure &&
+		  nzbInfo->GetUnpackStatus() != NzbInfo::usSpace &&
+		  nzbInfo->GetUnpackStatus() != NzbInfo::usPassword) ||
+		 (nzbInfo->GetUnpackStatus() == NzbInfo::usSuccess &&
+		  nzbInfo->GetParStatus() != NzbInfo::psFailure) ||
+		 ((nzbInfo->GetUnpackStatus() == NzbInfo::usNone ||
+		   nzbInfo->GetUnpackStatus() == NzbInfo::usSkipped) &&
+		  (nzbInfo->GetParStatus() == NzbInfo::psNone ||
+		   nzbInfo->GetParStatus() == NzbInfo::psSkipped) &&
+		  nzbInfo->CalcHealth() == 1000));
 
 	bool moveInter = !unpack &&
-		postInfo->GetNzbInfo()->GetMoveStatus() == NzbInfo::msNone &&
-		postInfo->GetNzbInfo()->GetUnpackStatus() != NzbInfo::usFailure &&
-		postInfo->GetNzbInfo()->GetUnpackStatus() != NzbInfo::usSpace &&
-		postInfo->GetNzbInfo()->GetUnpackStatus() != NzbInfo::usPassword &&
-		postInfo->GetNzbInfo()->GetParStatus() != NzbInfo::psFailure &&
-		postInfo->GetNzbInfo()->GetParStatus() != NzbInfo::psManual &&
-		postInfo->GetNzbInfo()->GetDeleteStatus() == NzbInfo::dsNone &&
+		nzbInfo->GetMoveStatus() == NzbInfo::msNone &&
+		nzbInfo->GetUnpackStatus() != NzbInfo::usFailure &&
+		nzbInfo->GetUnpackStatus() != NzbInfo::usSpace &&
+		nzbInfo->GetUnpackStatus() != NzbInfo::usPassword &&
+		nzbInfo->GetParStatus() != NzbInfo::psFailure &&
+		nzbInfo->GetParStatus() != NzbInfo::psManual &&
+		nzbInfo->GetDeleteStatus() == NzbInfo::dsNone &&
+		!(((nzbInfo->GetUnpackStatus() == NzbInfo::usNone ||
+		   nzbInfo->GetUnpackStatus() == NzbInfo::usSkipped) &&
+		  (nzbInfo->GetParStatus() == NzbInfo::psNone ||
+		   nzbInfo->GetParStatus() == NzbInfo::psSkipped) &&
+		  nzbInfo->CalcHealth() < 1000)) &&
 		!Util::EmptyStr(g_Options->GetInterDir()) &&
-		!strncmp(postInfo->GetNzbInfo()->GetDestDir(), g_Options->GetInterDir(), strlen(g_Options->GetInterDir())) &&
-		postInfo->GetNzbInfo()->GetDestDir()[strlen(g_Options->GetInterDir())] == PATH_SEPARATOR;
+		!strncmp(nzbInfo->GetDestDir(), g_Options->GetInterDir(), strlen(g_Options->GetInterDir())) &&
+		nzbInfo->GetDestDir()[strlen(g_Options->GetInterDir())] == PATH_SEPARATOR;
 
 	if (unpack && parFailed)
 	{
-		postInfo->GetNzbInfo()->PrintMessage(Message::mkWarning,
-			"Skipping unpack for %s due to %s", postInfo->GetNzbInfo()->GetName(),
-			postInfo->GetNzbInfo()->GetParStatus() == NzbInfo::psManual ? "required par-repair" : "par-failure");
-		postInfo->GetNzbInfo()->SetUnpackStatus(NzbInfo::usSkipped);
+		nzbInfo->PrintMessage(Message::mkWarning,
+			"Skipping unpack for %s due to %s", nzbInfo->GetName(),
+			nzbInfo->GetParStatus() == NzbInfo::psManual ? "required par-repair" : "par-failure");
+		nzbInfo->SetUnpackStatus(NzbInfo::usSkipped);
 		unpack = false;
 	}
 
+	bool postUnpackRenaming = g_Options->GetRenameAfterUnpack() &&
+		nzbInfo->GetPostUnpackRenamingStatus() == NzbInfo::PostUnpackRenamingStatus::None &&
+		nzbInfo->GetDestDir() &&
+		nzbInfo->GetName() &&
+		nzbInfo->GetUnpackStatus() != NzbInfo::usFailure &&
+		nzbInfo->GetUnpackStatus() != NzbInfo::usSpace &&
+		nzbInfo->GetUnpackStatus() != NzbInfo::usPassword &&
+		nzbInfo->GetParStatus() != NzbInfo::psFailure &&
+		nzbInfo->GetParStatus() != NzbInfo::psManual &&
+		nzbInfo->GetMoveStatus() == NzbInfo::msSuccess;
+
 	if (unpack)
 	{
 		EnterStage(downloadQueue, postInfo, PostInfo::ptUnpacking);
@@ -774,6 +871,11 @@ void PrePostProcessor::StartJob(DownloadQueue* downloadQueue, PostInfo* postInfo
 		EnterStage(downloadQueue, postInfo, PostInfo::ptMoving);
 		MoveController::StartJob(postInfo);
 	}
+	else if (postUnpackRenaming)
+	{
+		EnterStage(downloadQueue, postInfo, PostInfo::ptPostUnpackRenaming);
+		PostUnpackRenamer::Controller::StartJob(postInfo);
+	}
 	else
 	{
 		EnterStage(downloadQueue, postInfo, PostInfo::ptExecutingScript);
@@ -798,6 +900,7 @@ void PrePostProcessor::JobCompleted(DownloadQueue* downloadQueue, PostInfo* post
 		NzbCompleted(downloadQueue, nzbInfo, false);
 	}
 
+	Guard guard(m_waitMutex);
 	m_queuedJobs--;
 }
 
@@ -833,16 +936,16 @@ void PrePostProcessor::UpdatePauseState()
 		}
 	}
 
-	if (needPause && !g_Options->GetTempPauseDownload())
+	if (needPause && !g_WorkState->GetTempPauseDownload())
 	{
 		info("Pausing download before post-processing");
 	}
-	else if (!needPause && g_Options->GetTempPauseDownload())
+	else if (!needPause && g_WorkState->GetTempPauseDownload())
 	{
 		info("Unpausing download after post-processing");
 	}
 
-	g_Options->SetTempPauseDownload(needPause);
+	g_WorkState->SetTempPauseDownload(needPause);
 }
 
 bool PrePostProcessor::EditList(DownloadQueue* downloadQueue, IdList* idList,
@@ -927,7 +1030,7 @@ void PrePostProcessor::FileDownloaded(DownloadQueue* downloadQueue, NzbInfo* nzb
 		g_QueueScriptCoordinator->EnqueueScript(nzbInfo, QueueScriptCoordinator::qeFileDownloaded);
 	}
 
-	if (g_Options->GetDirectUnpack() && g_Options->GetDecode())
+	if (g_Options->GetDirectUnpack() && !g_Options->GetRawArticle() && !g_Options->GetSkipWrite() && !nzbInfo->GetSkipDiskWrite())
 	{
 		bool allowPar;
 		if (nzbInfo->GetDirectUnpackStatus() == NzbInfo::nsNone &&
@@ -935,7 +1038,7 @@ void PrePostProcessor::FileDownloaded(DownloadQueue* downloadQueue, NzbInfo* nzb
 			DirectUnpack::IsArchiveFilename(fileInfo->GetFilename()) &&
 			CanRunMoreJobs(&allowPar))
 		{
-			NzbParameter* unpackParameter = nzbInfo->GetParameters()->Find("*Unpack:", false);
+			NzbParameter* unpackParameter = nzbInfo->GetParameters()->Find("*Unpack:");
 			bool wantUnpack = !(unpackParameter && !strcasecmp(unpackParameter->GetValue(), "no"));
 			if (wantUnpack && nzbInfo->GetFailedArticles() == 0)
 			{
