@@ -2,7 +2,7 @@
  *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2007-2017 Andrey Prygunkov <hugbug@users.sourceforge.net>
- *  Copyright (C) 2023-2024 Denis <denis@nzbget.com>
+ *  Copyright (C) 2023-2025 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -238,25 +238,6 @@ void Util::SplitInt64(int64 Int64, uint32* Hi, uint32* Lo)
 	*Lo = (uint32)(Int64 & 0xFFFFFFFF);
 }
 
-std::optional<double> 
-Util::StrToNum(const std::string& str)
-{
-	std::istringstream ss(str);
-	double num;
-
-	if (ss >> num)
-	{
-		if (!ss.eof()) 
-		{
-			return std::nullopt;
-		}
-
-		return { num };
-	}
-
-	return std::nullopt;
-}
-
 /* Base64 decryption is taken from
 *  Article "BASE 64 Decoding and Encoding Class 2003" by Jan Raddatz
 *  http://www.codeguru.com/cpp/cpp/algorithms/article.php/c5099/
@@ -526,9 +507,7 @@ void Util::TrimRight(char* str)
 
 void Util::TrimRight(std::string& str)
 {
-	while (
-		!str.empty() &&
-		(str.back() == '\n' || str.back() == '\r' || str.back() == ' ' || str.back() == '\t'))
+	while (!str.empty() && std::isspace(str.back()))
 	{
 		str.pop_back();
 	}
