@@ -1,7 +1,7 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
- *  Copyright (C) 2014-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2014-2019 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "nzbget.h"
 #include "Log.h"
 #include "Options.h"
+#include "WorkState.h"
 #include "FeedCoordinator.h"
 #include "StatMeter.h"
 #include "WinConsole.h"
@@ -29,14 +30,11 @@
 #include "Util.h"
 #include "resource.h"
 
-extern Options* g_Options;
 extern char* (*g_Arguments)[];
 extern int g_ArgumentCount;
 extern void ExitProc();
 extern void Reload();
 extern WinConsole* g_WinConsole;
-extern FeedCoordinator* g_FeedCoordinator;
-extern StatMeter* g_StatMeter;
 
 #define UM_TRAYICON (WM_USER + 1)
 #define UM_QUIT (WM_USER + 2)
@@ -62,7 +60,7 @@ BOOL WINAPI WinConsole::ConsoleCtrlHandler(DWORD dwCtrlType)
 			ExitProc();
 			while (g_WinConsole)
 			{
-				usleep(20 * 1000);
+				Util::Sleep(20);
 			}
 			return TRUE;
 
@@ -71,29 +69,11 @@ BOOL WINAPI WinConsole::ConsoleCtrlHandler(DWORD dwCtrlType)
 	}
 }
 
-WinConsole::WinConsole()
-{
-	m_initialArguments = nullptr;
-	m_initialArgumentCount = 0;
-	m_defaultArguments = nullptr;
-	m_iconData = nullptr;
-	m_modal = false;
-	m_autostart = false;
-	m_showTrayIcon = true;
-	m_showConsole = false;
-	m_showWebUI = true;
-	m_autoParam = false;
-	m_doubleClick = false;
-	m_trayWindow = 0;
-	m_running = false;
-	m_runningService = false;
-}
-
 WinConsole::~WinConsole()
 {
 	if (m_initialArguments)
 	{
-		g_Arguments =	(char*(*)[])m_initialArguments;
+		g_Arguments = (char*(*)[])m_initialArguments;
 		g_ArgumentCount = m_initialArgumentCount;
 	}
 	free(m_defaultArguments);
@@ -107,7 +87,6 @@ void WinConsole::InitAppMode()
 	m_instance = (HINSTANCE)GetModuleHandle(0);
 	DWORD processId;
 	GetWindowThreadProcessId(GetConsoleWindow(), &processId);
-	m_appMode = false;
 
 	if (GetCurrentProcessId() == processId && g_ArgumentCount == 1)
 	{
@@ -131,24 +110,27 @@ void WinConsole::InitAppMode()
 			{
 				break;
 			}
-			if (!strcmp((*g_Arguments)[i], "-app"))
+			else if (!strcmp((*g_Arguments)[i], "-app"))
 			{
 				m_appMode = true;
 			}
-
-			if (!strcmp((*g_Arguments)[i], "-auto"))
+			else if (!strcmp((*g_Arguments)[i], "-auto"))
 			{
 				m_autoParam = true;
 			}
+			else if (!strcmp((*g_Arguments)[i], "-A"))
+			{
+				m_addParam = true;
+			}
 		}
 
-		if (m_appMode)
+		if (m_appMode || m_autoParam)
 		{
 			m_initialArguments = (char**)g_Arguments;
 			m_initialArgumentCount = g_ArgumentCount;
 
 			// remove "-app" from command line
-			int argc = g_ArgumentCount - 1 - (m_autoParam ? 1 : 0);
+			int argc = g_ArgumentCount - (m_appMode ? 1 : 0) - (m_autoParam ? 1 : 0);
 			m_defaultArguments = (char**)malloc(sizeof(char*) * (argc + 2));
 
 			int p = 0;
@@ -166,7 +148,13 @@ void WinConsole::InitAppMode()
 		}
 	}
 
-	// m_bAppMode indicates whether the program was started as a standalone app
+	if (m_addParam)
+	{
+		RunAnotherInstance();
+		return;
+	}
+
+	// m_appMode indicates whether the program was started as a standalone app
 	// (not from a dos box window). In that case we hide the console window,
 	// show the tray icon and start in server mode
 
@@ -179,7 +167,7 @@ void WinConsole::InitAppMode()
 
 void WinConsole::Run()
 {
-	if (!m_appMode)
+	if (!m_appMode || m_addParam)
 	{
 		return;
 	}
@@ -208,7 +196,7 @@ void WinConsole::Run()
 		}
 		else
 		{
-			usleep(20 * 1000);
+			Util::Sleep(20);
 			counter += 20;
 			if (counter >= 200)
 			{
@@ -300,10 +288,10 @@ LRESULT WinConsole::TrayWndProc(HWND hwndWin, UINT uMsg, WPARAM wParam, LPARAM l
 		case UM_TRAYICON:
 			if (lParam == WM_LBUTTONUP && !m_doubleClick)
 			{
-				g_Options->SetPauseDownload(!g_Options->GetPauseDownload());
-				g_Options->SetPausePostProcess(g_Options->GetPauseDownload());
-				g_Options->SetPauseScan(g_Options->GetPauseDownload());
-				g_Options->SetResumeTime(0);
+				g_WorkState->SetPauseDownload(!g_WorkState->GetPauseDownload());
+				g_WorkState->SetPausePostProcess(g_WorkState->GetPauseDownload());
+				g_WorkState->SetPauseScan(g_WorkState->GetPauseDownload());
+				g_WorkState->SetResumeTime(0);
 				UpdateTrayIcon();
 			}
 			else if (lParam == WM_LBUTTONDBLCLK && m_doubleClick)
@@ -317,6 +305,7 @@ LRESULT WinConsole::TrayWndProc(HWND hwndWin, UINT uMsg, WPARAM wParam, LPARAM l
 			return 0;
 
 		case UM_QUIT:
+		case WM_ENDSESSION:
 			ExitProc();
 			return 0;
 
@@ -378,15 +367,15 @@ void WinConsole::ShowMenu()
 			}
 
 		case ID_INFO_HOMEPAGE:
-			ShellExecute(0, "open", "http://nzbget.net", nullptr, nullptr, SW_SHOWNORMAL);
+			ShellExecute(0, "open", "https://nzbget.com", nullptr, nullptr, SW_SHOWNORMAL);
 			break;
 
 		case ID_INFO_DOWNLOADS:
-			ShellExecute(0, "open", "http://nzbget.net/download", nullptr, nullptr, SW_SHOWNORMAL);
+			ShellExecute(0, "open", "https://nzbget.com/download/", nullptr, nullptr, SW_SHOWNORMAL);
 			break;
 
 		case ID_INFO_FORUM:
-			ShellExecute(0, "open", "http://nzbget.net/forum", nullptr, nullptr, SW_SHOWNORMAL);
+			ShellExecute(0, "open", "https://nzbget.com/forum/", nullptr, nullptr, SW_SHOWNORMAL);
 			break;
 
 		case ID_ABOUT:
@@ -466,12 +455,12 @@ void WinConsole::ShowAboutBox()
 	m_modal = false;
 }
 
-BOOL CALLBACK WinConsole::AboutDialogProcStat(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK WinConsole::AboutDialogProcStat(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	return g_WinConsole->AboutDialogProc(hwndDlg, uMsg, wParam, lParam);
 }
 
-BOOL WinConsole::AboutDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR WinConsole::AboutDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	switch(uMsg)
 	{
@@ -494,7 +483,7 @@ BOOL WinConsole::AboutDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM
 			}
 			else if (LOWORD(wParam) == IDC_ABOUT_HOMEPAGE && HIWORD(wParam) == STN_CLICKED)
 			{
-				ShellExecute(0, "open", "http://nzbget.net", nullptr, nullptr, SW_SHOWNORMAL);
+				ShellExecute(0, "open", "https://nzbget.com", nullptr, nullptr, SW_SHOWNORMAL);
 			}
 			else if (LOWORD(wParam) == IDC_ABOUT_GPL && HIWORD(wParam) == STN_CLICKED)
 			{
@@ -517,7 +506,7 @@ BOOL WinConsole::AboutDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM
 				(HWND)wParam == GetDlgItem(hwndDlg, IDC_ABOUT_GPL))
 			{
 				SetCursor(m_handCursor);
-				SetWindowLong(hwndDlg, DWL_MSGRESULT, TRUE);
+				SetWindowLongPtr(hwndDlg, DWLP_MSGRESULT, TRUE);
 				return TRUE;
 			}
 			return FALSE;
@@ -539,12 +528,12 @@ void WinConsole::ShowPrefsDialog()
 	m_modal = false;
 }
 
-BOOL CALLBACK WinConsole::PrefsDialogProcStat(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK WinConsole::PrefsDialogProcStat(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	return g_WinConsole->PrefsDialogProc(hwndDlg, uMsg, wParam, lParam);
 }
 
-BOOL WinConsole::PrefsDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR WinConsole::PrefsDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	switch(uMsg)
 	{
@@ -671,6 +660,7 @@ void WinConsole::LoadPrefs()
 void WinConsole::ApplyPrefs()
 {
 	ShowWindow(GetConsoleWindow(), m_showConsole ? SW_SHOW : SW_HIDE);
+	g_WorkState->SetPauseFrontend(!m_showConsole);
 	if (m_showTrayIcon)
 	{
 		UpdateTrayIcon();
@@ -701,7 +691,7 @@ void WinConsole::ShowRunningDialog()
 	HWND hTrayWindow = FindWindow("NZBGet tray window", nullptr);
 	m_running = true;
 
-	int result = DialogBox(m_instance, MAKEINTRESOURCE(IDD_RUNNINGDIALOG), m_trayWindow, RunningDialogProcStat);
+	INT_PTR result = DialogBox(m_instance, MAKEINTRESOURCE(IDD_RUNNINGDIALOG), m_trayWindow, RunningDialogProcStat);
 
 	switch (result)
 	{
@@ -720,12 +710,12 @@ void WinConsole::ShowRunningDialog()
 	}
 }
 
-BOOL CALLBACK WinConsole::RunningDialogProcStat(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK WinConsole::RunningDialogProcStat(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	return g_WinConsole->RunningDialogProc(hwndDlg, uMsg, wParam, lParam);
 }
 
-BOOL WinConsole::RunningDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR WinConsole::RunningDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	switch(uMsg)
 	{
@@ -770,12 +760,12 @@ void WinConsole::UpdateTrayIcon()
 	strncpy(oldTip, m_iconData->szTip, sizeof(m_iconData->szTip));
 	oldTip[200-1] = '\0';
 
-	if (g_Options->GetPauseDownload())
+	if (g_WorkState->GetPauseDownload())
 	{
 		m_iconData->hIcon = m_pausedIcon;
 		strncpy(m_iconData->szTip, "NZBGet - paused", sizeof(m_iconData->szTip));
 	}
-	else if (!g_StatMeter->GetStandBy())
+	else if (g_WorkState->GetDownloading())
 	{
 		m_iconData->hIcon = m_workingIcon;
 		BString<100> tip("NZBGet - downloading at %s", *Util::FormatSpeed(g_StatMeter->CalcCurrentDownloadSpeed()));
@@ -923,7 +913,7 @@ void WinConsole::ShowFactoryResetDialog()
 	HWND hTrayWindow = FindWindow("NZBGet tray window", nullptr);
 	m_running = true;
 
-	int result = DialogBox(m_instance, MAKEINTRESOURCE(IDD_FACTORYRESETDIALOG), m_trayWindow, FactoryResetDialogProcStat);
+	INT_PTR result = DialogBox(m_instance, MAKEINTRESOURCE(IDD_FACTORYRESETDIALOG), m_trayWindow, FactoryResetDialogProcStat);
 
 	switch (result)
 	{
@@ -933,12 +923,12 @@ void WinConsole::ShowFactoryResetDialog()
 	}
 }
 
-BOOL CALLBACK WinConsole::FactoryResetDialogProcStat(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK WinConsole::FactoryResetDialogProcStat(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	return g_WinConsole->FactoryResetDialogProc(hwndDlg, uMsg, wParam, lParam);
 }
 
-BOOL WinConsole::FactoryResetDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR WinConsole::FactoryResetDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	switch(uMsg)
 	{
@@ -965,8 +955,8 @@ void WinConsole::ResetFactoryDefaults()
 	BString<1024> path;
 	CString errmsg;
 
-	g_Options->SetPauseDownload(true);
-	g_Options->SetPausePostProcess(true);
+	g_WorkState->SetPauseDownload(true);
+	g_WorkState->SetPausePostProcess(true);
 
 	char commonAppDataPath[MAX_PATH];
 	SHGetFolderPath(nullptr, CSIDL_COMMON_APPDATA, nullptr, 0, commonAppDataPath);
@@ -982,7 +972,7 @@ void WinConsole::ResetFactoryDefaults()
 		while (retry > 0 && FileSystem::DirectoryExists(path) &&
 			!FileSystem::DeleteDirectoryWithContent(path, errmsg))
 		{
-			usleep(200 * 1000);
+			Util::Sleep(200);
 			retry--;
 		}
 
@@ -1043,3 +1033,12 @@ void WinConsole::ResetFactoryDefaults()
 	mayStartBrowser = true;
 	Reload();
 }
+
+void WinConsole::RunAnotherInstance()
+{
+	ShowWindow(GetConsoleWindow(), SW_HIDE);
+	if (!(FindWindow("NZBGet tray window", nullptr) || IsServiceRunning()))
+	{
+		ShellExecute(0, "open", (*g_Arguments)[0], nullptr, nullptr, SW_SHOWNORMAL);
+	}
+}
