@@ -1,7 +1,8 @@
 /*
- *  This file is part of nzbget. See <http://nzbget.net>.
+ *  This file is part of nzbget. See <https://nzbget.com>.
  *
  *  Copyright (C) 2015-2016 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2023 Denis <denis@nzbget.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,13 +15,12 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-
 #include "nzbget.h"
 
-#include "catch.h"
+#include <boost/test/unit_test.hpp>
 
 #include "FeedFilter.h"
 
@@ -32,11 +32,11 @@ void TestFilter(FeedItemInfo* feedItemInfo, const char* filterDef, FeedItemInfo:
 	FeedFilter filter(filterDef);
 	filter.Match(*feedItemInfo);
 
-	INFO(filterDef);
-	REQUIRE(feedItemInfo->GetMatchStatus() == expectedMatch);
+	BOOST_TEST_MESSAGE(filterDef);
+	BOOST_TEST(feedItemInfo->GetMatchStatus() == expectedMatch);
 }
 
-TEST_CASE("Feed filter: one liners", "[FeedFilter][Quick]")
+BOOST_AUTO_TEST_CASE(FeedFilterTest)
 {
 	FeedItemInfo item;
 	item.SetTitle("Game.of.Clowns.S02E06.REAL.1080p.HDTV.X264-Group.WEB-DL");
